# The purpose of this script is to make Supplementary Tables 3 and 4
# of the submitted manuscript.

# We read in the output produced by gen_sim_data_suppfig3.R and gen_sim_data_suppfig4.R
# and set up the table.

# You will need to move the 40 files generated by gen_omni_data_fig2.R into your data directory
# to run this script.

# Also make sure all the jobs from gen_sim_data_supfigg3.R and gen_sim_data_suppfig4.R have finished running,
# otherwise the necessary data will not exist and there will be errors.

########################################################################################
# Change this location to the directory that holds the necessary R libraries
# (must be installed first).
libs <- '/n/home/user/Rlibrary/3.3.1'

# Change data_dir your working directory directory which holds all the necessary data files.
data_dir <- '/users/user/desktop'

########################################################################################
# No manual changes need to be made after this point
########################################################################################

########################################################################################
# Load library
library(mvtnorm, lib=libs)
library(dplyr, lib=libs)

# Build the new parameter matrix
rho1_vec <- c(0, 0.3)
rho2_vec <- c(0, 0.3)
rho3_vec <- c(0, 0.3)
d_vec <- c(100)
param_mat <- expand.grid(rho1_vec, rho3_vec, rho2_vec, d_vec)
colnames(param_mat) <- c('rho1', 'rho2', 'rho3', 'd')

# Read in simulation results, attach parameter matrix
param_results <- c()
for (Snum in c(1,2,6,8)) {
    for (i in 1:50) {
        temp_fname <- paste('power_results_S', Snum, '_', i, 'betaconstant.txt', sep='')
        temp_file <- tryCatch(read.table(temp_fname, header=T, sep='\t'), error=function(e) e, warning=function(w) w)
        if (length(class(temp_file)) > 1) {
            next
            cat('Missing S ', Snum, ' aID ', aID, '\n')
        }
        temp_file <- temp_file %>% mutate(Snum = Snum)
        
        param_results <- rbind(param_results, temp_file)
    }
}
dim(param_results)


########################################################################################
# This table will hold the first ncausal to reach 80% power at 
# constant beta for each simulation setting
alpha <- 0.01
power_mat <- matrix(data=NA, nrow=10, ncol=9)
power_mat <- data.frame(power_mat)
colnames(power_mat) <- c('rho1', 'rho2', 'rho3', 'd', 
                         'GBJ', 'GHC',
                         'minP', 'skat', 'omni')

# This table holds temporary powers for each Snum
temp_mat <- data.frame(ncausal=1:10, GBJ=NA, GHC=NA,
                       minP=NA, skat=NA, omni=NA, BJ=NA)

# Loop through four simulation settings
for (Snum in c(1,2,6,8)) {
    for (i in 1:50)
    {
        temp_data <- param_results[which(param_results$ncausal==i & 
                                             param_results$Snum == Snum), ]
        
        # Read in the omnibus correlation matrix file
        d <- param_mat$d[Snum]
        rho1 <- param_mat$rho1[[Snum]]
        rho2 <- param_mat$rho2[[Snum]]
        rho3 <- param_mat$rho3[[Snum]]
        temp_omni_fname <- paste('d', d, '_rho1_', 10*rho1, '_rho2_', 10*rho2, '_rho3_', 
                                 10*rho3, '_ncausal_', i, '.txt', sep='')
        temp_omni_mat <- read.table(temp_omni_fname, header=T)
        
        # Get the omni test statistics
        omni_pvec <- rep(NA, nrow(temp_data))
        for (j in 1:nrow(temp_data)) {
            omni_stat <- temp_data$omni_stat[j] 
            omni_bound <- tryCatch(qnorm(1-omni_stat), warning=function(w) w, error=function(e) e)
            omni_Z_bounds <- rep(omni_bound, 4)
            omni_pvec[j] <- 1 - pmvnorm(lower=rep(-Inf, 4), upper=omni_Z_bounds,
                                        sigma=unname(as.matrix(temp_omni_mat)))[1]
        }
        
        # Record power
        temp_mat$GBJ[i] <- sum(temp_data$GBJ_p<alpha) / nrow(temp_data)
        temp_mat$GHC[i] <- sum(temp_data$GHC_p<alpha) / nrow(temp_data)
        temp_mat$minP[i] <- sum(temp_data$minP_pvalue<alpha) / nrow(temp_data)
        temp_mat$skat[i] <- sum(temp_data$skat_p<alpha) / nrow(temp_data)
        temp_mat$omni[i]<- sum(omni_pvec<alpha) / nrow(temp_data)
        temp_mat$BJ[i] <- sum(temp_data$BJ_p<alpha) / nrow(temp_data)
    }
    
    # Report parameters
    power_mat$rho1[Snum] <- temp_data$rho1[1]
    power_mat$rho2[Snum] <- temp_data$rho2[1]
    power_mat$rho3[Snum] <- temp_data$rho3[1]
    power_mat$d[Snum] <- temp_data$d[1]
    power_mat$GBJ[Snum] <- min(which(temp_mat$GBJ >= 0.8))
    power_mat$GHC[Snum] <- min(which(temp_mat$GHC >= 0.8))
    power_mat$minP[Snum] <- min(which(temp_mat$minP >= 0.8))
    power_mat$skat[Snum] <- min(which(temp_mat$skat >= 0.8))
    power_mat$omni[Snum] <- min(which(temp_mat$omni >= 0.8))
}

# This produces Supplementary Table 3
power_mat %>% filter(!is.na(rho1))


########################################################################################
# Now for Supplementary Table 4

# Read in simulation results, attach parameter matrix
param_results <- c()
for (Snum in c(1,2,6,8)) {
    for (i in 1:20) {
        temp_fname <- paste('power_results_S', Snum, '_', i, 'beta15.txt', sep='')
        temp_file <- tryCatch(read.table(temp_fname, header=T, sep='\t'), error=function(e) e, warning=function(w) w)
        if (length(class(temp_file)) > 1) {
            next
            cat('Missing S ', Snum, ' aID ', aID, '\n')
        }
        temp_file <- temp_file %>% mutate(Snum = Snum)
        
        param_results <- rbind(param_results, temp_file)
    }
}

########################################################################################
# This table will hold the first ncausal to reach 80% power at 
# constant beta for each simulation setting
alpha <- 0.01
power_mat <- matrix(data=NA, nrow=10, ncol=9)
power_mat <- data.frame(power_mat)
colnames(power_mat) <- c('rho1', 'rho2', 'rho3', 'd', 
                         'GBJ', 'GHC',
                         'minP', 'skat', 'omni')

# This table holds temporary powers for each Snum
temp_mat <- data.frame(ncausal=1:10, GBJ=NA, GHC=NA,
                       minP=NA, skat=NA, omni=NA, BJ=NA)

# Loop through four simulation settings
for (Snum in c(1,2,6,8)) {
    for (i in 1:10)
    {
        temp_data <- param_results[which(param_results$ncausal==i & 
                                             param_results$Snum == Snum), ]
        
        # Read in the omnibus correlation matrix file
        d <- param_mat$d[Snum]
        rho1 <- param_mat$rho1[[Snum]]
        rho2 <- param_mat$rho2[[Snum]]
        rho3 <- param_mat$rho3[[Snum]]
        temp_omni_fname <- paste('d', d, '_rho1_', 10*rho1, '_rho2_', 10*rho2, '_rho3_', 
                                 10*rho3, '_ncausal_', i, '.txt', sep='')
        temp_omni_mat <- read.table(temp_omni_fname, header=T)
        
        # Get the omni test statistics
        omni_pvec <- rep(NA, nrow(temp_data))
        for (j in 1:nrow(temp_data)) {
            omni_stat <- temp_data$omni_stat[j] 
            omni_bound <- tryCatch(qnorm(1-omni_stat), warning=function(w) w, error=function(e) e)
            omni_Z_bounds <- rep(omni_bound, 4)
            omni_pvec[j] <- 1 - pmvnorm(lower=rep(-Inf, 4), upper=omni_Z_bounds,
                                        sigma=unname(as.matrix(temp_omni_mat)))[1]
        }
        
        # Record power
        temp_mat$GBJ[i] <- sum(temp_data$GBJ_p<alpha) / nrow(temp_data)
        temp_mat$GHC[i] <- sum(temp_data$GHC_p<alpha) / nrow(temp_data)
        temp_mat$minP[i] <- sum(temp_data$minP_pvalue<alpha) / nrow(temp_data)
        temp_mat$skat[i] <- sum(temp_data$skat_p<alpha) / nrow(temp_data)
        temp_mat$omni[i]<- sum(omni_pvec<alpha) / nrow(temp_data)
        temp_mat$BJ[i] <- sum(temp_data$BJ_p<alpha) / nrow(temp_data)
    }
    
    # Report parameters
    power_mat$rho1[Snum] <- temp_data$rho1[1]
    power_mat$rho2[Snum] <- temp_data$rho2[1]
    power_mat$rho3[Snum] <- temp_data$rho3[1]
    power_mat$d[Snum] <- temp_data$d[1]
    power_mat$GBJ[Snum] <- min(which(temp_mat$GBJ >= 0.8))
    power_mat$GHC[Snum] <- min(which(temp_mat$GHC >= 0.8))
    power_mat$minP[Snum] <- min(which(temp_mat$minP >= 0.8))
    power_mat$skat[Snum] <- min(which(temp_mat$skat >= 0.8))
    power_mat$omni[Snum] <- min(which(temp_mat$omni >= 0.8))
}

# This produces Supplementary Table 4
power_mat %>% filter(!is.na(rho1))